SorhaQuestLog = LibStub("AceAddon-3.0"):NewAddon("SorhaQuestLog", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SorhaQuestLog")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local LSM = LibStub("LibSharedMedia-3.0") 

-- Used to stop the need to use self.db all the time
local db
local dbChar

-- Just bringing some frequently used global functions into local scope
local pairs = pairs
local tinsert = tinsert
local tContains = tContains
local tremove = tremove
local select = select
local format = format
local SecondsToTime = SecondsToTime
local GetTime = GetTime

-- Register framd font
LSM:Register("font", "framd", [[Interface\AddOns\SorhaQuestLog\Fonts\framd.ttf]])

-- Binding globals for bindings.xml
BINDING_HEADER_SORHAQUESTLOG = "Sorha Quest Log";
BINDING_NAME_TOGGLE_SORHAQUESTLOG = L["Toggle Minion"]

--Frames
local fraSQLMainAnchor = nil
local fraQuestTimerAnchor = nil
local dicButtons = {}
local intNumberOfFrames = 50
local intNumberOfTimerFrames = 5
local strButtonPrefix = "SorhaQuestLogButton"

--Colour strings
local strInfoColour = "|cffffffff"
local strHeaderColour = "|cffffffff"
local strQuestLevelColour = "|cffffffff"
local strQuestTitleColour = "|cffffffff"
local strObjectiveTitleColour = "|cffffffff"
local strObjectiveStatusColour = "|cffffffff"
local strObjective00to24Colour = "|cffffffff"
local strObjective25to49Colour = "|cffffffff"
local strObjective50to74Colour = "|cffffffff"
local strObjective75to99Colour = "|cffffffff"
local strObjective100Colour = "|cffffffff"
local strObjectiveDescriptionColour = "|cffffffff"

-- Booleans
local blnInitialized = false -- Main minion frames created
local blnTimerInitialized = false -- Timer frames created
local blnIsUpdating = false -- Minion is updating
local blnWasAClick = false -- Was QUEST_LOG_UPDATE called by a click on a header etc
local blnHaveCleanedUp = false -- Have we removed old hidden quests
local blnIgnoreUpdateEvents = false -- Ignores QLU events, used when making large scale collapse/expands to log
local blnHadNoTimers = true -- Stop clearing timer frames if there was no timers last time 
local blnFirstUpdate = true -- Was first update of quest log, no old data will be on store yet

-- Strings used to store current location for auto collapse/expand
local strZone = ""
local strSubZone = ""

-- Current and previous tables of all questlog data
local curQuestInfo = nil
local oldQuestInfo = nil


-- Time stores
local intLastOnUpdateTime = 0 -- Time OnUpdate event was last used
local intTimeOfLastSound = 0 -- Time last sound played

-- Tables for select menus in options
local dicRepLevels = {
	["Hated"] = {["MTitle"] = FACTION_STANDING_LABEL1, ["FTitle"] = FACTION_STANDING_LABEL1_FEMALE, ["Value"] = 1.00},
	["Hostile"] = {["MTitle"] = FACTION_STANDING_LABEL2, ["FTitle"] = FACTION_STANDING_LABEL2_FEMALE, ["Value"] = 1.30},
	["Unfriendly"] = {["MTitle"] = FACTION_STANDING_LABEL3, ["FTitle"] = FACTION_STANDING_LABEL3_FEMALE, ["Value"] = 1.70},
	["Neutral"] = {["MTitle"] = FACTION_STANDING_LABEL4, ["FTitle"] = FACTION_STANDING_LABEL4_FEMALE, ["Value"] = 2.20},
	["Friendly"] = {["MTitle"] = FACTION_STANDING_LABEL5, ["FTitle"] = FACTION_STANDING_LABEL5_FEMALE, ["Value"] = 2.85},
	["Honored"] = {["MTitle"] = FACTION_STANDING_LABEL6, ["FTitle"] = FACTION_STANDING_LABEL6_FEMALE, ["Value"] = 3.71},
	["Revered"] = {["MTitle"] = FACTION_STANDING_LABEL7, ["FTitle"] = FACTION_STANDING_LABEL7_FEMALE, ["Value"] = 4.82},
	["Exalted"] = {["MTitle"] = FACTION_STANDING_LABEL8, ["FTitle"] = FACTION_STANDING_LABEL8_FEMALE, ["Value"] = 6.27},
}

local dicOutlines = {
	[""] = L["None"],
	["OUTLINE"] = L["Outline"],
	["THICKOUTLINE"] = L["Thick Outline"],
}

local dicQuestTitleColourOptions = {
	["Custom"] = L["Custom"],
	["Level"] = L["Level"],
	["Completion"] = L["Completion"],
	["Done/Undone"] = L["Done/Undone"],
}

local dicObjectiveColourOptions = {
	["Custom"] = L["Custom"],
	["Done/Undone"] = L["Done/Undone"],
	["Completion"] = L["Completion"],
}

local dicQuestTags = {
	[ELITE] = "+",
	[GROUP] = "g",
	[PVP] = "p",
	[RAID] = "r",
	[LFG_TYPE_DUNGEON] = "d",
	[PLAYER_DIFFICULTY2] = "d+",
	["Daily"] = "*",
}

local dicLongQuestTags = {
	[ELITE] = ELITE,
	[GROUP] = GROUP,
	[PVP] = PVP,
	[RAID] = RAID,
	[LFG_TYPE_DUNGEON] = LFG_TYPE_DUNGEON,
	[PLAYER_DIFFICULTY2] = PLAYER_DIFFICULTY2,
	["Daily"] = L["Daily"],
}


local defaults = { -- Defaults Table
	profile = {
		Main = {
			HideSorhaQuestLog = false,
			HideBlizzardTracker = false,
			ShowQuestTimerFrame = false,
		},
		Minion = {
			MinionWindowLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			MinionWindowScale = 1,
			MinionWindowLocked = false,			
			MinionWidth = 220,
			ShowItemButtons = true,
			ShowNumberOfQuests = false,
			MoveItemsAndTooltipsRight = false,
			CollapseToLeft = false,
			ItemButtonScale = 0.8,
		},
		Fonts = {
			QuestFontSize = 11,
			QuestFont = "framd",
			QuestFontOutline = "",
			ObjectiveFontSize = 11,
			ObjectiveFont = "framd",
			ObjectiveFontOutline = "",
			HeaderFontSize = 11,
			HeaderFont = "framd",
			HeaderFontOutline = "",
		},
		ZonesAndQuests = {
			QuestLevelColouringSetting = "Level",
			QuestTitleColouringSetting = "Level",
			ObjectiveTitleColouringSetting = "Custom",
			ObjectiveStatusColouringSetting = "Completion",
			FullLengthQuestTags = false,
			HideCompletedObjectives = false,
			ShowDescWhenNoObjectives = false,
			ShowAllQuests = false,
			AllowHiddenQuests = false,
			CollapseOnLeave = false,
			ExpandOnEnter = false,
			QuestHeadersHideWhenEmpty = false,
			ShowHiddenCountOnZones = false,
		},
		Sounds = {
			UseQuestDoneSound = false,
			UseObjectiveDoneSound = false,
		},
		Colours = {
			InfoColour = {r = 0, g = 1, b = 0.5, a = 1},
			HeaderColour = {r = 0, g = 0.6, b = 1, a = 1},
			QuestLevelColour = {r = 1, g = 1, b = 1, a = 1},
			QuestTitleColour = {r = 1, g = 1, b = 1, a = 1},
			ObjectiveTitleColour = {r = 1, g = 1, b = 1, a = 1},
			ObjectiveStatusColour = {r = 1, g = 1, b = 1, a = 1},
			Objective00PlusColour = {r = 1, g = 0, b = 0, a = 1},
			Objective25PlusColour = {r = 1, g = 0.3, b = 0, a = 1},
			Objective50PlusColour  = {r = 1, g = 0.6, b = 0, a = 1},
			Objective75PlusColour = {r = 1, g = 0.95, b = 0, a = 1},
			ObjectiveDoneColour = {r = 0, g = 1, b = 0, a = 1},
			ObjectiveDescColour = {r = 0.5, g = 0.5, b = 0.5, a = 0.5},
		},		
		Timer = {
			TimerWindowLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			TimerWindowScale = 1,
			TimerWindowLocked = false,
		},
	},
	char = {
		Quests = {
			HiddenQuests = {},
		},
	},
}

local function getOptions() -- Creates Options table.
	if not SorhaQuestLog.options then
		SorhaQuestLog.options = {
			type = "group",
			name = "SorhaQuestLog",
			handler = SorhaQuestLog,
			args = {
				General = {
					name = L["General Options"],
					type = "group",
					order = 1,
					args = {
						MainHide = {
							name = L["Hide Minion"],
							desc = L["Hides/Shows the Main Frame"],
							type = "toggle",
							get = function() return db.Main.HideSorhaQuestLog end,
							set = function()
										SorhaQuestLog:ToggleSorhaQuestLog()
									end,
							order = 1,
						},
						MainLock = {
							name = L["Lock Frame"],
							desc = L["Unlocks/Locks the Main Frame"],
							type = "toggle",
							get = function() return db.Minion.MinionWindowLocked end,
							set = function()
										db.Minion.MinionWindowLocked = not db.Minion.MinionWindowLocked
										SorhaQuestLog:doAnchorUpdates(false)
									end,
							order = 2,
						},
						HeaderMainFrameSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 10,
						},
						HeaderMainFrame = {
							name = L["Main Frame Settings"],
							type = "header",
							order = 11,
						},
						MainWidth = {
							order = 12,
							name = L["Width"],
							desc = L["Controls the width of the main window."],
							type = "range",
							min = 150, max = 300, step = 1,
							isPercent = false,
							get = function() return db.Minion.MinionWidth end,
							set = function(info, value)
								db.Minion.MinionWidth = value
								fraSQLMainAnchor:SetWidth(db.Minion.MinionWidth)
								SorhaQuestLog:UpdateMinion()
							end,
						},	
						Reset = {
							order = 13,
							type = "execute",
							name = L["Reset Main Frame"],
							desc = L["Resets Main Frame position"],
							func = function()
								db.Minion.MinionWindowLocation.Point = "CENTER"
								db.Minion.MinionWindowLocation.RelativePoint =  "CENTER"
								db.Minion.MinionWindowLocation.X = 0
								db.Minion.MinionWindowLocation.Y = 0
								fraSQLMainAnchor:StopMovingOrSizing()		
								fraSQLMainAnchor:ClearAllPoints()
								fraSQLMainAnchor:SetPoint(db.Minion.MinionWindowLocation.Point, UIParent, db.Minion.MinionWindowLocation.RelativePoint, db.Minion.MinionWindowLocation.X, db.Minion.MinionWindowLocation.Y)
							end,
						},
						HeaderMiscSettingsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 30,
						},
						HeaderMiscSettings = {
							name = L["Misc. Settings"],
							type = "header",
							order = 31,
						},
						ShowNumberQuestsToggle = {
							name = L["Show number of quests"],
							desc = L["Shows/Hides the number of quests"],
							type = "toggle",
							get = function() return db.Minion.ShowNumberOfQuests end,
							set = function()
								db.Minion.ShowNumberOfQuests = not db.Minion.ShowNumberOfQuests
								SorhaQuestLog:UpdateMinion()
							end,
							order = 32,
						},
						CollapseToLeftToggle = {
							name = L["Autoshrink to left"],
							desc = L["Shrinks the width down when the length of current quests is less then the max width\nNote: Doesn't work well with quests that wordwrap"],
							type = "toggle",
							get = function() return db.Minion.CollapseToLeft end,
							set = function()
								db.Minion.CollapseToLeft = not db.Minion.CollapseToLeft
								SorhaQuestLog:UpdateMinion()
							end,
							order = 33,
						},
						ItemsAndTooltipsRightToggle = {
							name = L["Display items and tooltips on right"],
							desc = L["Moves items and tooltips to the right"],
							type = "toggle",
							width = "full",
							get = function() return db.Minion.MoveItemsAndTooltipsRight end,
							set = function()
								db.Minion.MoveItemsAndTooltipsRight = not db.Minion.MoveItemsAndTooltipsRight
								SorhaQuestLog:UpdateMinion()
							end,
							order = 34,
						},
						HeaderBlizzardFrameSettingsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 50,
						},
						HeaderBlizzardFrameSettings = {
							name = L["Blizzard Frame Settings"],
							type = "header",
							order = 51,
						},
						HideBlizzardTrackerToggle = {
							name = L["Hide the default quest tracker"],
							desc = L["Hides blizzards quest tracker.. which is also used for achivement tracking"],
							type = "toggle",
							width = "full",
							get = function() return db.Main.HideBlizzardTracker end,
							set = function()
								db.Main.HideBlizzardTracker = not db.Main.HideBlizzardTracker
								SorhaQuestLog:HandleBlizzardTracker()
							end,
							order = 52,
						},
						ShowQuestTimerFrameToggle = {
							name = L["Show Quest Timer Frame"],
							desc = L["Shows a quest timer frame since the blizzard one hides with the default watcher"],
							type = "toggle",
							get = function() return db.Main.ShowQuestTimerFrame end,
							set = function()
								if (db.Main.ShowQuestTimerFrame == false) then
									if (blnTimerInitialized == false) then
										SorhaQuestLog:CreateTimerFrame()
									end
								end
								db.Main.ShowQuestTimerFrame = not db.Main.ShowQuestTimerFrame
								SorhaQuestLog:doAnchorUpdates(false)
							end,
							order = 53,
						},
						TimerWindowLockedToggle = {
							name = L["Lock Quest Timer Frame"],
							desc = L["Locks the quest timer frame"],
							type = "toggle",
							disabled = function() return not db.Main.ShowQuestTimerFrame end,
							get = function() return db.Timer.TimerWindowLocked end,
							set = function()
								db.Timer.TimerWindowLocked = not db.Timer.TimerWindowLocked
								SorhaQuestLog:doAnchorUpdates(false)
							end,
							order = 54,
						},
						HeaderItemButtonsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 70,
						},
						HeaderItemButtons = {
							name = L["Item Button Settings"],
							type = "header",
							order = 71,
						},
						ShowItemButtonsToggle = {
							name = L["Show quest item buttons"],
							desc = L["Shows/Hides the quest item buttons"],
							type = "toggle",
							width = "full",
							get = function() return db.Minion.ShowItemButtons end,
							set = function()
								db.Minion.ShowItemButtons = not db.Minion.ShowItemButtons
								SorhaQuestLog:UpdateMinion()
							end,
							order = 72,
						},
						ItemButtonsSizeSlider = {
							order = 73,
							name = L["Item Button Size"],
							desc = L["Controls the size of the Item Buttons."],
							type = "range",
							disabled = function() return not(db.Minion.ShowItemButtons) end,
							min = 0.5, max = 2, step = 0.05,
							isPercent = false,
							get = function() return db.Minion.ItemButtonScale end,
							set = function(info, value)
								db.Minion.ItemButtonScale = value
								SorhaQuestLog:doItemButtonSizeChange()
							end,
						},
					},
				},
				QuestOptions = {
					name = L["Quest Options"],
					type = "group",
					order = 2,
					args = {
						DisplaySettingsHeader = {
							name = L["Display Settings"],
							type = "header",
							order = 1,
						},
						AllowHiddenQuestsToggle = {
							name = L["Allow quests to be hidden"],
							desc = L["Allows quests to be hidden and enables the show/hide button"],
							type = "toggle",
							width = "full",
							get = function() return db.ZonesAndQuests.AllowHiddenQuests end,
							set = function()
								db.ZonesAndQuests.AllowHiddenQuests = not db.ZonesAndQuests.AllowHiddenQuests
								SorhaQuestLog:doHiddenQuestsUpdate()
							end,
							order = 3,
						},
						AllowHiddenHeadersToggle = {
							name = L["Zone headers hide when all contained quests are hidden"],
							desc = L["Makes zone headers hide when all contained quests are hidden"],
							type = "toggle",
							disabled = function() return not db.ZonesAndQuests.AllowHiddenQuests end,
							width = "full",
							get = function() return db.ZonesAndQuests.QuestHeadersHideWhenEmpty end,
							set = function()
								db.ZonesAndQuests.QuestHeadersHideWhenEmpty = not db.ZonesAndQuests.QuestHeadersHideWhenEmpty
								SorhaQuestLog:UpdateMinion()
							end,
							order = 4,
						},		
						AllowHiddenCountOnZonesToggle = {
							name = L["Display count of hidden quest in each zone"],
							desc = L["Displays a count of the hidden quests in each zone on the zone header"],
							type = "toggle",
							disabled = function() return not db.ZonesAndQuests.AllowHiddenQuests end,
							width = "full",
							get = function() return db.ZonesAndQuests.ShowHiddenCountOnZones end,
							set = function()
								db.ZonesAndQuests.ShowHiddenCountOnZones = not db.ZonesAndQuests.ShowHiddenCountOnZones
								SorhaQuestLog:UpdateMinion()
							end,
							order = 5,
						},
						FullLengthQuestTagsToggle = {
							name = L["Display full length quest tags"],
							desc = L["Displays full length quest tags like 'Elite' instead of '+'"],
							type = "toggle",
							width = "full",
							get = function() return db.ZonesAndQuests.FullLengthQuestTags end,
							set = function()
								db.ZonesAndQuests.FullLengthQuestTags = not db.ZonesAndQuests.FullLengthQuestTags
								SorhaQuestLog:UpdateMinion()
							end,
							order = 6,
						},						
						HeaderZoneHeadersSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 20,
						},
						HeaderZoneHeaders = {
							name = L["Zone Header Settings"],
							type = "header",
							order = 21,
						},
						ExpandOnEnterToggle = {
							name = L["Auto expand zones on enter"],
							desc = L["Automatically expands zone headers when you enter the zone"],
							type = "toggle",
							get = function() return db.ZonesAndQuests.ExpandOnEnter end,
							set = function()
								db.ZonesAndQuests.ExpandOnEnter = not db.ZonesAndQuests.ExpandOnEnter
								SorhaQuestLog:doHandleZoneChange()
							end,
							order = 22,
						},	
						CollapseOnLeaveToggle = {
							name = L["Auto collapse zones on exit"],
							desc = L["Automatically collapses zone headers when you exit the zone"],
							type = "toggle",
							get = function() return db.ZonesAndQuests.CollapseOnLeave end,
							set = function()
								db.ZonesAndQuests.CollapseOnLeave = not db.ZonesAndQuests.CollapseOnLeave
								SorhaQuestLog:doHandleZoneChange()
							end,
							order = 23,
						},	
						HeaderFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 24,
							name = L["Header Font"],
							desc = L["The font that the zone headers will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.HeaderFont end,
							set = function(info, value)
								db.Fonts.HeaderFont = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						HeaderFontOutlineSelect = {
							name = L["Header Font Outline"],
							desc = L["The outline that the zone headers will use"],
							type = "select",
							order = 25,
							values = dicOutlines,
							get = function() return db.Fonts.HeaderFontOutline end,
							set = function(info, value)
								db.Fonts.HeaderFontOutline = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						HeaderFontSize = {
							order = 26,
							name = L["Header Font Size"],
							desc = L["Controls the font size of the header label."],
							type = "range",
							min = 10, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.HeaderFontSize end,
							set = function(info, value)
								db.Fonts.HeaderFontSize = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						HeaderQuestsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 40,
						},
						HeaderQuests = {
							name = L["Quest Settings"],
							type = "header",
							order = 41,
						},
						HideCompletedObjectivesToggle = {
							name = L["Hide completed objectives"],
							desc = L["Shows/Hides completed objectives"],
							type = "toggle",
							width = "full",
							get = function() return db.ZonesAndQuests.HideCompletedObjectives end,
							set = function()
								db.ZonesAndQuests.HideCompletedObjectives = not db.ZonesAndQuests.HideCompletedObjectives
								SorhaQuestLog:UpdateMinion()
							end,
							order = 42,
						},
						ShowDescWhenNoObjectivesToggle = {
							name = L["Display quest description if not objectives"],
							desc = L["Displays a quests description if there are no objectives available"],
							type = "toggle",
							width = "full",
							get = function() return db.ZonesAndQuests.ShowDescWhenNoObjectives end,
							set = function()
								db.ZonesAndQuests.ShowDescWhenNoObjectives = not db.ZonesAndQuests.ShowDescWhenNoObjectives
								SorhaQuestLog:UpdateMinion()
							end,
							order = 43,
						},	
						QuestFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 44,
							name = L["Quest Font"],
							desc = L["The font that the quest titles will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.QuestFont end,
							set = function(info, value)
								db.Fonts.QuestFont = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						QuestFontOutlineSelect = {
							name = L["Quest Font Outline"],
							desc = L["The outline that the quest titles will use"],
							type = "select",
							order = 45,
							values = dicOutlines,
							get = function() return db.Fonts.QuestFontOutline end,
							set = function(info, value)
								db.Fonts.QuestFontOutline = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						QuestFontSize = {
							order = 46,
							name = L["Quest Font Size"],
							desc = L["Controls the font size of the quest info."],
							type = "range",
							min = 10, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.QuestFontSize end,
							set = function(info, value)
								db.Fonts.QuestFontSize = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
					},
				},
				ColourOptions = {
					name = L["Colour Options"],
					type = "group",
					order = 3,
					args = {
						HeaderColourSettings = {
							name = L["Colour Settings"],
							type = "header",
							order = 1,
						},
						QuestLevelColouringSelect = {
							name = L["Colour quest levels by:"],
							desc = L["The setting by which the colour of quest levels are determined"],
							type = "select",
							order = 2,
							values = dicQuestTitleColourOptions,
							get = function() return db.ZonesAndQuests.QuestLevelColouringSetting end,
							set = function(info, value)
								db.ZonesAndQuests.QuestLevelColouringSetting = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						QuestTitleColouringSelect = {
							name = L["Colour quest titles by:"],
							desc = L["The setting by which the colour of quest titles is determined"],
							type = "select",
							order = 3,
							values = dicQuestTitleColourOptions,
							get = function() return db.ZonesAndQuests.QuestTitleColouringSetting end,
							set = function(info, value)
								db.ZonesAndQuests.QuestTitleColouringSetting = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						ObjectiveTitleColouringSelect = {
							name = L["Colour objective title text by:"],
							desc = L["The setting by which the colour of objective title is determined"],
							type = "select",
							order = 4,
							values = dicObjectiveColourOptions,
							get = function() return db.ZonesAndQuests.ObjectiveTitleColouringSetting end,
							set = function(info, value)
								db.ZonesAndQuests.ObjectiveTitleColouringSetting = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						ObjectiveStatusColouringSelect = {
							name = L["Colour objective status text by:"],
							desc = L["The setting by which the colour of objective statuses is determined"],
							type = "select",
							order = 5,
							values = dicObjectiveColourOptions,
							get = function() return db.ZonesAndQuests.ObjectiveStatusColouringSetting end,
							set = function(info, value)
								db.ZonesAndQuests.ObjectiveStatusColouringSetting = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						HeaderMainColoursSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 9,
						},
						HeaderMainColours = {
							name = L["Main Colours"],
							type = "header",
							order = 10,
						},						
						HeaderColour = {
							name = L["Zone Header Colour"],
							desc = L["Sets the color for the header of each zone"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.HeaderColour.r, db.Colours.HeaderColour.g, db.Colours.HeaderColour.b, db.Colours.HeaderColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.HeaderColour.r = r
									db.Colours.HeaderColour.g = g
									db.Colours.HeaderColour.b = b
									db.Colours.HeaderColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 11,
						},
						InfoTextColour = {
							name = L["Info Text"],
							desc = L["Sets the color of the info text (Title bar, # of quests hidden etc)"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.InfoColour.r, db.Colours.InfoColour.g, db.Colours.InfoColour.b, db.Colours.InfoColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.InfoColour.r = r
									db.Colours.InfoColour.g = g
									db.Colours.InfoColour.b = b
									db.Colours.InfoColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 12,
						},
						QuestLevelColour = {
							name = L["Quest levels"],
							desc = L["Sets the color for the quest levels if custom colouring is on"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.QuestLevelColouringSetting == "Custom") end,
							hasAlpha = true,
							get = function() return db.Colours.QuestLevelColour.r, db.Colours.QuestLevelColour.g, db.Colours.QuestLevelColour.b, db.Colours.QuestLevelColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.QuestLevelColour.r = r
									db.Colours.QuestLevelColour.g = g
									db.Colours.QuestLevelColour.b = b
									db.Colours.QuestLevelColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 13,
						},
						QuestTitleColour = {
							name = L["Quest titles"],
							desc = L["Sets the color for the quest titles if colouring by level is off"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.QuestTitleColouringSetting == "Custom") end,
							hasAlpha = true,
							get = function() return db.Colours.QuestTitleColour.r, db.Colours.QuestTitleColour.g, db.Colours.QuestTitleColour.b, db.Colours.QuestTitleColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.QuestTitleColour.r = r
									db.Colours.QuestTitleColour.g = g
									db.Colours.QuestTitleColour.b = b
									db.Colours.QuestTitleColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 14,
						},
						ObjectiveTitleColourPicker = {
							name = L["Objective title colour"],
							desc = L["Sets the custom color for objectives titles"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Custom") end,
							hasAlpha = true,
							get = function() return db.Colours.ObjectiveTitleColour.r, db.Colours.ObjectiveTitleColour.g, db.Colours.ObjectiveTitleColour.b, db.Colours.ObjectiveTitleColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ObjectiveTitleColour.r = r
									db.Colours.ObjectiveTitleColour.g = g
									db.Colours.ObjectiveTitleColour.b = b
									db.Colours.ObjectiveTitleColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 15,
						},
						ObjectiveStatusColourPicker = {
							name = L["Objective status colour"],
							desc = L["Sets the custom color for objectives statuses"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Custom") end,
							hasAlpha = true,
							get = function() return db.Colours.ObjectiveStatusColour.r, db.Colours.ObjectiveStatusColour.g, db.Colours.ObjectiveStatusColour.b, db.Colours.ObjectiveStatusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ObjectiveStatusColour.r = r
									db.Colours.ObjectiveStatusColour.g = g
									db.Colours.ObjectiveStatusColour.b = b
									db.Colours.ObjectiveStatusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 16,
						},
						NoObjectivesColour = {
							name = L["No objectives description colour"],
							desc = L["Sets the color for the description displayed when there is no quest objectives"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.ObjectiveDescColour.r, db.Colours.ObjectiveDescColour.g, db.Colours.ObjectiveDescColour.b, db.Colours.ObjectiveDescColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ObjectiveDescColour.r = r
									db.Colours.ObjectiveDescColour.g = g
									db.Colours.ObjectiveDescColour.b = b
									db.Colours.ObjectiveDescColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 17,
						},
						HeaderGradualColoursSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 30,
						},
						HeaderGradualColours = {
							name = L["Gradual objective Colours"],
							type = "header",
							order = 31,
						},
						Objective00PlusColour = {
							name = L["0% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 0% complete"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.Objective00PlusColour.r, db.Colours.Objective00PlusColour.g, db.Colours.Objective00PlusColour.b, db.Colours.Objective00PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective00PlusColour.r = r
									db.Colours.Objective00PlusColour.g = g
									db.Colours.Objective00PlusColour.b = b
									db.Colours.Objective00PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 32,
						},
						Objective25PlusColour = {
							name = L["25% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 25% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = true,
							get = function() return db.Colours.Objective25PlusColour.r, db.Colours.Objective25PlusColour.g, db.Colours.Objective25PlusColour.b, db.Colours.Objective25PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective25PlusColour.r = r
									db.Colours.Objective25PlusColour.g = g
									db.Colours.Objective25PlusColour.b = b
									db.Colours.Objective25PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 33,
						},
						Objective50PlusColour = {
							name = L["50% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 50% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = false,
							get = function() return db.Colours.Objective50PlusColour.r, db.Colours.Objective50PlusColour.g, db.Colours.Objective50PlusColour.b, db.Colours.Objective50PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective50PlusColour.r = r
									db.Colours.Objective50PlusColour.g = g
									db.Colours.Objective50PlusColour.b = b
									db.Colours.Objective50PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 34,
						},
						Objective75PlusColour = {
							name = L["75% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 75% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = true,
							get = function() return db.Colours.Objective75PlusColour.r, db.Colours.Objective75PlusColour.g, db.Colours.Objective75PlusColour.b, db.Colours.Objective75PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective75PlusColour.r = r
									db.Colours.Objective75PlusColour.g = g
									db.Colours.Objective75PlusColour.b = b
									db.Colours.Objective75PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 35,
						},
						DoneObjectiveColour = {
							name = L["Complete objective colour"],
							desc = L["Sets the color for the complete objectives"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.ObjectiveDoneColour.r, db.Colours.ObjectiveDoneColour.g, db.Colours.ObjectiveDoneColour.b, db.Colours.ObjectiveDoneColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ObjectiveDoneColour.r = r
									db.Colours.ObjectiveDoneColour.g = g
									db.Colours.ObjectiveDoneColour.b = b
									db.Colours.ObjectiveDoneColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 36,
						},
					},
				},
				SoundOptions = {
					name = L["Sound Options"],
					type = "group",
					order = 4,
					args = {
						DisplaySettingsHeader = {
							name = L["Sound Settings"],
							type = "header",
							order = 1,
						},
						UseQuestDoneSoundToggle = {
							name = L["Enable quest completed sound"],
							desc = L["Play sound when all of a quests objectives are completed"],
							type = "toggle",
							width = "full",
							get = function() return db.Sounds.UseQuestDoneSound end,
							set = function()
								db.Sounds.UseQuestDoneSound = not db.Sounds.UseQuestDoneSound
							end,
							order = 2,
						},						
						UseObjectiveDoneSoundToggle = {
							name = L["Enable objective completed sound"],
							desc = L["Play sound when a quests objective is completed"],
							type = "toggle",
							width = "full",
							get = function() return db.Sounds.UseObjectiveDoneSound end,
							set = function()
								db.Sounds.UseObjectiveDoneSound = not db.Sounds.UseObjectiveDoneSound
							end,
							order = 3,
						},
					},
				},
			},
		}
		SorhaQuestLog.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(SorhaQuestLog.db)
		SorhaQuestLog.options.args.profiles.order = 9
	end
	return SorhaQuestLog.options
	
end

function SorhaQuestLog:OnInitialize() -- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("SorhaQuestLogDB", defaults, true)
	db = self.db.profile
	dbChar = self.db.char
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
	self:SetupOptions()
	self:SetupColours()
end

function SorhaQuestLog:OnEnable() -- Called when the addon is enabled
	strZone = GetRealZoneText()
	strSubZone = GetSubZoneText()
	self:CreateLayout()

	self:RegisterEvent("QUEST_LOG_UPDATE")	
	self:RegisterEvent('PLAYER_LEVEL_UP');
	self:RegisterEvent("ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_INDOORS")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	
	hooksecurefunc("WatchFrame_Update",function()
		self:HandleBlizzardTracker()
	end);
	
	intTimeSinceLastSound = GetTime()
	blnInitialized = true
	LSM.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateMedia")
end

function SorhaQuestLog:UpdateMedia(event, mediatype, key) -- LSM getting new media
	if mediatype == "font" then
        if key == db.Fonts.QuestFont or key == db.Fonts.HeaderFont then 
			if (blnIsUpdating == false) then
				self:UpdateMinion()
			end
		end
    end
end

function SorhaQuestLog:QUEST_LOG_UPDATE(event, arg1)
	if (db.Main.HideSorhaQuestLog == false) then
		if (blnWasAClick == true) then -- Was the event called from a click (Skips the delay on header collapses etc)
			blnIsUpdating = true
			self:UpdateMinion()
		elseif (blnIsUpdating == false) then --If not updating then update, forces a 0.5 second delay between system called updates
			if (blnIgnoreUpdateEvents == false) then
				blnIsUpdating = true
				self:ScheduleTimer("UpdateMinion", 0.25)
			end
		end
	end
	
	-- If not cleaned dead hidden quests do so
	if (blnHaveCleanedUp == false) then
		blnHaveCleanedUp = true
		self:CleanupHiddenQuestList()
	end
end

function SorhaQuestLog:PLAYER_LEVEL_UP(...)
	if (db.Main.HideSorhaQuestLog == false) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:ZONE_CHANGED(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:ZONE_CHANGED_INDOORS(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:ZONE_CHANGED_NEW_AREA(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:OnUpdate()
	-- If half a second is past and timer shown, (then if timer not started create), then refresh
	if ((GetTime() - intLastOnUpdateTime) > 0.5) then
		intLastOnUpdateTime = GetTime()
		if (blnInitialized == true and db.Main.ShowQuestTimerFrame == true) then
			if (blnTimerInitialized == false) then
				SorhaQuestLog:CreateTimerFrame()
			end
			SorhaQuestLog:RefreshTimers(GetQuestTimers())
		end
	end
end

function SorhaQuestLog:RefreshTimers(...) -- Refresh timer minion
	local numTimers = select("#", ...);
	local dblYPosition = db.Fonts.HeaderFontSize
	local dblOffset = 0
		
	-- Adds needed buttons for timers and sets text/size etc
	for i = 1, numTimers do
		fraQuestTimerAnchor.Buttons[i]:ClearAllPoints()
		fraQuestTimerAnchor.Buttons[i]:SetPoint("TOPLEFT", fraQuestTimerAnchor, "TOPLEFT", 0, -dblYPosition);
		fraQuestTimerAnchor.Buttons[i].objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestFont), db.Fonts.QuestFontSize, db.Fonts.QuestFontOutline)
		fraQuestTimerAnchor.Buttons[i].objFontString:SetText(" - " .. SecondsToTime(select(i, ...)))
		dblOffset = fraQuestTimerAnchor.Buttons[i].objFontString:GetStringHeight()
		
		fraQuestTimerAnchor.Buttons[i]:SetHeight(dblOffset)
		fraQuestTimerAnchor.Buttons[i]:SetWidth(fraQuestTimerAnchor.Buttons[i].objFontString:GetStringWidth())
		
		fraQuestTimerAnchor.Buttons[i]:SetScript("OnEnter", function (self) GameTooltip:SetOwner(self); GameTooltip:SetHyperlink(GetQuestLink(GetQuestIndexForTimer(i))); GameTooltip:Show(); end);
		fraQuestTimerAnchor.Buttons[i]:SetScript("OnLeave", GameTooltip_Hide);
		fraQuestTimerAnchor.Buttons[i]:EnableMouse(true);
		fraQuestTimerAnchor.Buttons[i]:Show()
		
		dblYPosition = dblYPosition + dblOffset
	end

	-- If there were timers last cycle for each unused timer button clean it up
	if (blnHadNoTimers == false) then
		for i = numTimers + 1, intNumberOfTimerFrames, 1 do
			fraQuestTimerAnchor.Buttons[i].objFontString:SetText("")
			fraQuestTimerAnchor.Buttons[i]:SetScript("OnEnter", nil);
			fraQuestTimerAnchor.Buttons[i]:SetScript("OnLeave", nil);
			fraQuestTimerAnchor.Buttons[i]:EnableMouse(false);
			fraQuestTimerAnchor.Buttons[i]:Hide()
		end
	end
	
	-- Show/hide header text based on timer count, and store number of timers this cycle
	if (numTimers > 0) then
		blnHadNoTimers = false
		fraQuestTimerAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), db.Fonts.HeaderFontSize, db.Fonts.HeaderFontOutline)
		fraQuestTimerAnchor.objFontString:SetText(strInfoColour .. "Quest Timers")
	else
		blnHadNoTimers = true
		fraQuestTimerAnchor.objFontString:SetText("")
	end
end

function SorhaQuestLog:RefreshConfig()
	db = self.db.profile
	fraSQLMainAnchor:SetWidth(db.Minion.MinionWidth)
	self:doAnchorUpdates(true)
	self:doColourChange()
end

function SorhaQuestLog:CreateLayout() -- Create main minion and its buttons etc
	-- Main Anchor
	fraSQLMainAnchor = self:doCreateFrame("FRAME","SorhaQuestLogMainAnchor",UIParent,db.Minion.MinionWidth,40,1,"BACKGROUND",1, db.Minion.MinionWindowLocation.Point, UIParent, db.Minion.MinionWindowLocation.RelativePoint, db.Minion.MinionWindowLocation.X, db.Minion.MinionWindowLocation.Y, 1)
	fraSQLMainAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraSQLMainAnchor:SetMovable(true)
	fraSQLMainAnchor:SetClampedToScreen(true)
	fraSQLMainAnchor:RegisterForDrag("LeftButton")
	fraSQLMainAnchor:SetScript("OnDragStart", fraSQLMainAnchor.StartMoving)
	fraSQLMainAnchor:SetScript("OnUpdate", function(self) SorhaQuestLog:OnUpdate() end)
	fraSQLMainAnchor:SetScript("OnDragStop",  function(self)
		fraSQLMainAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraSQLMainAnchor:GetPoint()
		db.Minion.MinionWindowLocation.Point = strPoint
		db.Minion.MinionWindowLocation.RelativePoint = strRelativePoint
		db.Minion.MinionWindowLocation.X = intPosX
		db.Minion.MinionWindowLocation.Y = intPosY
	end)
	
	-- Number of quests fontstring
	fraSQLMainAnchor.objFontString = fraSQLMainAnchor:CreateFontString(nil, "OVERLAY");
	fraSQLMainAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 14, db.Fonts.HeaderFontOutline)
	fraSQLMainAnchor.objFontString:SetJustifyH("LEFT")
	fraSQLMainAnchor.objFontString:SetJustifyV("TOP")
	fraSQLMainAnchor.objFontString:SetText("");
	fraSQLMainAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	fraSQLMainAnchor.objFontString:SetShadowOffset(1, -1)
	
	-- Show/Hide hidden quest button
	fraSQLMainAnchor.buttonShowHidden = self:doCreateLooseFrame("BUTTON","SQLShowHiddenButton",fraSQLMainAnchor,16,16,1,"LOW",1,1)
	fraSQLMainAnchor.buttonShowHidden:SetPoint("TOPLEFT", fraSQLMainAnchor, "TOPLEFT", 0, -6);
	fraSQLMainAnchor.buttonShowHidden:SetBackdrop({bgFile="Interface\\AddOns\\SorhaQuestLog\\Textures\\button1.tga", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=0, tileSize=0, edgeSize=1, insets={left=-0, right=0, top=0, bottom=0}})
	fraSQLMainAnchor.buttonShowHidden:SetBackdropBorderColor(0, 0, 0, 0)
	
	-- Show/Hide hidden quest button hover
	fraSQLMainAnchor.buttonShowHiddenHover = self:doCreateLooseFrame("FRAME","SQLShowHiddenButtonHover",fraSQLMainAnchor.buttonShowHidden,fraSQLMainAnchor.buttonShowHidden:GetWidth(),fraSQLMainAnchor.buttonShowHidden:GetHeight(),1,"LOW",1,1)
	fraSQLMainAnchor.buttonShowHiddenHover:SetPoint("TOPLEFT", fraSQLMainAnchor.buttonShowHidden, "TOPLEFT", 0, 0);
	fraSQLMainAnchor.buttonShowHiddenHover:SetBackdrop({bgFile="Interface\\AddOns\\SorhaQuestLog\\Textures\\button1_hover.tga", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=0, tileSize=0, edgeSize=1, insets={left=-0, right=0, top=0, bottom=0}})
	fraSQLMainAnchor.buttonShowHiddenHover:SetBackdropColor(0, 0, 1, 1)
	fraSQLMainAnchor.buttonShowHiddenHover:SetBackdropBorderColor(0,0,0,0)
	fraSQLMainAnchor.buttonShowHiddenHover:SetAlpha(0)
	
	-- Show/Hide hidden quest button events
	fraSQLMainAnchor.buttonShowHidden:RegisterForClicks("AnyUp")
	fraSQLMainAnchor.buttonShowHidden:SetScript("OnEnter", function() fraSQLMainAnchor.buttonShowHiddenHover:SetAlpha(1) end)
	fraSQLMainAnchor.buttonShowHidden:SetScript("OnLeave", function() fraSQLMainAnchor.buttonShowHiddenHover:SetAlpha(0) end)
	fraSQLMainAnchor.buttonShowHidden:SetScript("OnClick", function() 
		db.ZonesAndQuests.ShowAllQuests = not db.ZonesAndQuests.ShowAllQuests
		if (db.ZonesAndQuests.ShowAllQuests == true) then
			fraSQLMainAnchor.buttonShowHidden:SetBackdropColor(0, 0.3, 1, 1)
		else
			fraSQLMainAnchor.buttonShowHidden:SetBackdropColor(0.5, 0.5, 0.5, 1)
		end
		self:UpdateMinion()
	end)
	
	
	self:doAnchorUpdates(false) -- Update backcolour and visibility
	self:doHiddenQuestsUpdate()	-- Update show/hide hidden quests button position/visibility etc
	
	local strButtonName = ""
	for i=1,intNumberOfFrames,1 do -- Create quest log buttons
		strButtonName = strButtonPrefix .. i
		dicButtons[strButtonPrefix .. i] = self:doCreateLooseFrame("BUTTON",strButtonName,fraSQLMainAnchor,10,10,1,"LOW",1,1)
		dicButtons[strButtonPrefix .. i]:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
		dicButtons[strButtonPrefix .. i].objFontString = dicButtons[strButtonPrefix .. i]:CreateFontString(nil, "OVERLAY");
		dicButtons[strButtonPrefix .. i].objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 11, db.Fonts.HeaderFontOutline)
		dicButtons[strButtonPrefix .. i].objFontString:SetPoint("LEFT", dicButtons[strButtonPrefix .. i], "LEFT", 0, 0);
		dicButtons[strButtonPrefix .. i].objFontString:SetJustifyH("LEFT")
		dicButtons[strButtonPrefix .. i].objFontString:SetJustifyV("TOP")
		dicButtons[strButtonPrefix .. i].objFontString:SetText("");
		dicButtons[strButtonPrefix .. i].objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		dicButtons[strButtonPrefix .. i].objFontString:SetShadowOffset(1, -1)
		dicButtons[strButtonPrefix .. i]:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
		dicButtons[strButtonPrefix .. i]:SetBackdropColor(0, 0, 0, 0)
		dicButtons[strButtonPrefix .. i]:SetBackdropBorderColor(0, 0, 0, 0)

		strButtonName = strButtonPrefix .. "item" .. i
		dicButtons[strButtonPrefix .. i].fraItemButton = CreateFrame("BUTTON", strButtonName, dicButtons[strButtonPrefix .. i], "WatchFrameItemButtonTemplate");
		dicButtons[strButtonPrefix .. i].fraItemButton:SetScale(db.Minion.ItemButtonScale)

		dicButtons[strButtonPrefix .. i].isHeader = nil
		dicButtons[strButtonPrefix .. i].isCollapsed = nil
		dicButtons[strButtonPrefix .. i].isUsed = false
		dicButtons[strButtonPrefix .. i].LogPosition = 0
		
		dicButtons[strButtonPrefix .. i]:RegisterForClicks("AnyUp")
		dicButtons[strButtonPrefix .. i]:SetScript("OnClick", function(self, button)
			blnWasAClick = true
			if (button == "LeftButton") then
				if (self.isHeader == 1) then
					if (self.isCollapsed == 1) then
						ExpandQuestHeader(self.LogPosition)
					else
						CollapseQuestHeader(self.LogPosition)
					end
				else
					if (IsShiftKeyDown()) then
						ChatEdit_InsertLink(GetQuestLink(self.LogPosition))
					else
						ShowUIPanel(QuestLogFrame);
						SelectQuestLogEntry(self.LogPosition);
						QuestLog_SetSelection(self.LogPosition);
						QuestLog_Update();
					end
				end
			else
				local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(self.LogPosition);
				if (self.isHeader == 1) then
					if (db.ZonesAndQuests.AllowHiddenQuests == true) then
						SorhaQuestLog:DisplayRightClickMenu(self)
					end
				else
					if (db.ZonesAndQuests.AllowHiddenQuests == true) then
						if not(tContains(dbChar.Quests.HiddenQuests, questID)) then
							tinsert(dbChar.Quests.HiddenQuests, questID)	
						else
							SorhaQuestLog:RemoveObject(dbChar.Quests.HiddenQuests, questID)
						end
						SorhaQuestLog:UpdateMinion()
					end
				end
			end
		end)
		dicButtons[strButtonPrefix .. i]:SetScript("OnEnter", function(self)
			if (not(self.isHeader)) then
				SelectQuestLogEntry(self.LogPosition);
				local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(self.LogPosition);
				if (db.Minion.MoveItemsAndTooltipsRight == true) then
					GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, -50);
				else 
					GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, -50);
				end
				
				local strQuestDescription, strQuestObjectives = GetQuestLogQuestText();
				local objColour = GetQuestDifficultyColor(level);
				GameTooltip:SetText(questTitle, 0, 1, 0, 1);
				GameTooltip:AddLine(strQuestObjectives, 0.5, 0.5, 0.5, 1);
				
				local strQuestTag = ""
				if (isDaily) then
					strQuestTag = strQuestTag .. L["Daily"] .. " "
				end				
				if (questTag) then
					strQuestTag = strQuestTag .. questTag
					if (suggestedGroup > 0) then
						strQuestTag = strQuestTag .. " (" .. suggestedGroup .. ")"
					end
				end				
				GameTooltip:AddLine(strQuestTag, 1, 0, 0, 1);
				
				local intPartyMembers = GetNumPartyMembers();
				local blnIsOnQuest = false
				local k = 1
				if (intPartyMembers > 0) then
					GameTooltip:AddLine(L["Party members on quest:"], 0, 1, 0, 1);
					for k = 1, intPartyMembers do
						blnIsOnQuest = IsUnitOnQuest(i, "party" .. k);
						if (blnIsOnQuest == 1) then
							GameTooltip:AddLine(UnitName("party" .. k), 1, 1, 1, 1);
						end
					end				
				end
				
				GameTooltip:Show();
			end
		end)
		dicButtons[strButtonPrefix .. i]:SetScript("OnLeave", function(self) 
			GameTooltip:Hide() 
		end)

	end
end

function SorhaQuestLog:CreateTimerFrame() -- Creates quest timer frame
	-- Timer anchor
	fraQuestTimerAnchor = self:doCreateFrame("FRAME","SQLQuestTimerAnchor",fraSQLMainAnchor,100,40,1,"BACKGROUND",1, db.Timer.TimerWindowLocation.Point, UIParent, db.Timer.TimerWindowLocation.RelativePoint, db.Timer.TimerWindowLocation.X, db.Timer.TimerWindowLocation.Y, 1)
	fraQuestTimerAnchor:SetMovable(true)
	fraQuestTimerAnchor:SetClampedToScreen(true)
	fraQuestTimerAnchor:RegisterForDrag("LeftButton")
	fraQuestTimerAnchor:SetScript("OnDragStart", fraQuestTimerAnchor.StartMoving)
	fraQuestTimerAnchor:SetScript("OnDragStop",  function(self)
		fraQuestTimerAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraQuestTimerAnchor:GetPoint()
		db.Timer.TimerWindowLocation.Point = strPoint
		db.Timer.TimerWindowLocation.RelativePoint = strRelativePoint
		db.Timer.TimerWindowLocation.X = intPosX
		db.Timer.TimerWindowLocation.Y = intPosY
	end)
	fraQuestTimerAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraQuestTimerAnchor:SetBackdropColor(0, 0, 1, 1)
	fraQuestTimerAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	-- Timer fontstring for title "Quest Timer"
	fraQuestTimerAnchor.objFontString = fraQuestTimerAnchor:CreateFontString(nil, "OVERLAY");
	fraQuestTimerAnchor.objFontString:SetPoint("TOPLEFT", fraQuestTimerAnchor, "TOPLEFT",0, 0);
	fraQuestTimerAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 14, db.Fonts.HeaderFontOutline)
	fraQuestTimerAnchor.objFontString:SetJustifyH("LEFT")
	fraQuestTimerAnchor.objFontString:SetJustifyV("TOP")
	fraQuestTimerAnchor.objFontString:SetText("");
	fraQuestTimerAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	fraQuestTimerAnchor.objFontString:SetShadowOffset(1, -1)
	fraQuestTimerAnchor.Buttons = {}
	
	local objButton = nil
	local strFramePrefix = "SQLTimerFrame"
	for i = 1,intNumberOfTimerFrames,1 do -- Create timer frames
		objButton = self:doCreateLooseFrame("FRAME",strFramePrefix .. i, fraQuestTimerAnchor, 100, 11, 1, "LOW", 1, 1)
		objButton:SetPoint("TOPLEFT", fraQuestTimerAnchor, "TOPLEFT", 0, -i*db.Fonts.QuestFontSize);
		objButton:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
		objButton:SetBackdropBorderColor(0, 0, 0, 0)
		objButton:SetBackdropColor(0, 0, 0, 0)
		objButton.objFontString = objButton:CreateFontString(nil, "OVERLAY");
		objButton.objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestFont), db.Fonts.QuestFontSize, db.Fonts.QuestFontOutline)
		objButton.objFontString:SetPoint("LEFT", objButton, "LEFT", 0, 0);
		objButton.objFontString:SetJustifyH("LEFT")
		objButton.objFontString:SetJustifyV("CENTER")
		objButton.objFontString:SetText("");
		objButton:Hide()
		tinsert(fraQuestTimerAnchor.Buttons, objButton)
	end
	blnTimerInitialized = true
	self:doAnchorUpdates(false)
end

function SorhaQuestLog:UpdateMinion()
	local intQuestOffset = 10
	local intSpacingIncrease = 0
	local intLargestWidth = 0
	local blnLargestWidthIsHeader = false
	local intHeaderOutlineOffset = 0
	local intQuestOutlineOffset = 0
	local intYPosition = 0
	blnWasAClick = false
	
	for i = 1, intNumberOfFrames, 1 do -- Reset buttons to unused state
		dicButtons[strButtonPrefix .. i]:Hide()
		dicButtons[strButtonPrefix .. i].fraItemButton:Hide()
		dicButtons[strButtonPrefix .. i].objFontString:SetText("")
		dicButtons[strButtonPrefix .. i].objFontString:SetWidth(0)
		dicButtons[strButtonPrefix .. i].objFontString:SetHeight(0)
		dicButtons[strButtonPrefix .. i]:ClearAllPoints()
		dicButtons[strButtonPrefix .. i].isUsed = false
		dicButtons[strButtonPrefix .. i].isHeader = nil
		dicButtons[strButtonPrefix .. i].isCollapsed = nil
		dicButtons[strButtonPrefix .. i].LogPosition = 0
		dicButtons[strButtonPrefix .. i]:SetBackdropColor(0, 0, 0, 0)
	end

	-- If this isn't the first get of quest info store the current info for later use
	if not(curQuestInfo == nil) then
		oldQuestInfo = curQuestInfo
		blnFirstUpdate = false
	end
	curQuestInfo = self:GetQuestLogInformation()

	-- Quest/Objective complete Sounds
	if (blnFirstUpdate == false and (db.Sounds.UseQuestDoneSound == true or db.Sounds.UseObjectiveDoneSound == true)) then
		if ((GetTime() - intTimeOfLastSound) > 1) then
			local strSound = self:CheckQuestStateChange()
			if (strSound == "QuestDone" and db.Sounds.UseQuestDoneSound == true) then
				PlaySoundFile("Sound\\Creature\\Peon\\PeonBuildingComplete1.wav");
				intTimeOfLastSound = GetTime()
			elseif (strSound == "ObjectiveDone" and db.Sounds.UseObjectiveDoneSound == true) then
				PlaySoundFile("Sound\\Creature\\Peasant\\PeasantWhat3.wav");
				intTimeOfLastSound = GetTime()
			else
				--Nothing
			end
		end
	end	
	
	--Add in slight offsets for outlined text to try stop overlap
	if (db.Fonts.HeaderFontOutline == "THICKOUTLINE") then
		intHeaderOutlineOffset = 3
	elseif (db.Fonts.HeaderFontOutline == "OUTLINE") then
		intHeaderOutlineOffset = 1
	end
	if (db.Fonts.QuestFontOutline == "THICKOUTLINE") then
		intQuestOutlineOffset = 1
	elseif (db.Fonts.QuestFontOutline == "OUTLINE") then
		intQuestOutlineOffset = 0.5
	end
		
	-- Number of quests title display
	if (db.Minion.ShowNumberOfQuests == true) then
		fraSQLMainAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 14, db.Fonts.HeaderFontOutline)
		fraSQLMainAnchor.objFontString:SetText(strInfoColour .. curQuestInfo["NoQuests"] .. "/25|r");
		if (fraSQLMainAnchor.objFontString:GetWidth() > intLargestWidth) then
			if (db.ZonesAndQuests.AllowHiddenQuests == true) then
				intLargestWidth = fraSQLMainAnchor.objFontString:GetWidth() + 20 -- Offset for the show/hide button pushing the fontstring accross
			else
				intLargestWidth = fraSQLMainAnchor.objFontString:GetWidth()
			end
			
		end
	else
		fraSQLMainAnchor.objFontString:SetText("");
	end
	fraSQLMainAnchor:SetWidth(db.Minion.MinionWidth)

	local intCurrentButton = 0
	local objButton = nil
	for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
		if not((ZoneInstance["NumHidden"] == ZoneInstance["NumQuests"]) and (ZoneInstance["HasHidden"] == true) and db.ZonesAndQuests.QuestHeadersHideWhenEmpty == true and db.ZonesAndQuests.ShowAllQuests == false and db.ZonesAndQuests.AllowHiddenQuests == true) then
			intCurrentButton = intCurrentButton + 1
			objButton = dicButtons[strButtonPrefix .. intCurrentButton]
			objButton.LogPosition = ZoneInstance["LogPosition"]
			objButton.isHeader = 1
			objButton.isCollapsed = ZoneInstance["Status"]
		
			objButton:Show()
			objButton.isUsed = true
			
			local strPrefix = strHeaderColour .. "- "
			if (ZoneInstance["Status"] == 1) then
				strPrefix = strHeaderColour .. "+ "
			end

			objButton:SetPoint("TOP", fraSQLMainAnchor, "CENTER", 0, intYPosition)
			objButton.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), db.Fonts.HeaderFontSize, db.Fonts.HeaderFontOutline)
			if (db.ZonesAndQuests.ShowHiddenCountOnZones == true and db.ZonesAndQuests.AllowHiddenQuests == true) then
				if (ZoneInstance["NumHidden"] > 0) then
					objButton.objFontString:SetText(strPrefix .. ZoneInstance["Title"] .. "|r (" .. strInfoColour .. ZoneInstance["NumHidden"] .. "/" .. ZoneInstance["NumQuests"] .." Hidden)|r");	
				else
					objButton.objFontString:SetText(strPrefix .. ZoneInstance["Title"] .. "|r");
				end
			else
				objButton.objFontString:SetText(strPrefix .. ZoneInstance["Title"] .. "|r");
			end
			
			objButton:SetWidth(db.Minion.MinionWidth)
			--
			if (objButton.objFontString:GetWidth() > intLargestWidth) then
				intLargestWidth = objButton.objFontString:GetWidth()
				blnLargestWidthIsHeader = true
			end
			--
			objButton.objFontString:SetWidth(db.Minion.MinionWidth)
			
			intSpacingIncrease = objButton.objFontString:GetHeight() + intHeaderOutlineOffset + 2
			objButton:SetHeight(intSpacingIncrease)
			intYPosition = intYPosition - intSpacingIncrease
					
			for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
				if (QuestInstance["IsHidden"] == false or db.ZonesAndQuests.AllowHiddenQuests == false or db.ZonesAndQuests.ShowAllQuests == true) then
					intCurrentButton = intCurrentButton + 1
					objButton = dicButtons[strButtonPrefix .. intCurrentButton]
					objButton.LogPosition = QuestInstance["LogPosition"]
					
					objButton:Show()
					objButton.isUsed = true
					local strQuestText = self:GetQuestText(QuestInstance)

					
					objButton:SetPoint("TOP", fraSQLMainAnchor, "CENTER", intQuestOffset, intYPosition)
					objButton.objFontString:SetFont(LSM:Fetch("font", db.Fonts.QuestFont), db.Fonts.QuestFontSize, db.Fonts.QuestFontOutline)
					objButton.objFontString:SetText(strQuestText);
					objButton:SetWidth(db.Minion.MinionWidth - intQuestOffset*2)

					if (objButton.objFontString:GetWidth() + intQuestOffset*2 > intLargestWidth) then
						intLargestWidth = objButton.objFontString:GetWidth() + intQuestOffset*2
						blnLargestWidthIsHeader = false
					end

					objButton.objFontString:SetWidth(db.Minion.MinionWidth - intQuestOffset*2)
					intSpacingIncrease = objButton.objFontString:GetHeight() + intQuestOutlineOffset
				
					if (db.Minion.ShowItemButtons == true) then
						if (QuestInstance["QuestItem"]) then
							local objItem = QuestInstance["QuestItem"]
							objButton.fraItemButton:ClearAllPoints();
							if (db.Minion.MoveItemsAndTooltipsRight == true) then
								objButton.fraItemButton:SetPoint("TOPLEFT", objButton, "TOPRIGHT", 3, 0)
							else
								objButton.fraItemButton:SetPoint("TOPRIGHT", objButton, "TOPLEFT", -8, 0)
							end
							
							objButton.fraItemButton:Show()
							objButton.fraItemButton:SetID(QuestInstance["LogPosition"])
							SetItemButtonTexture(objButton.fraItemButton, objItem["Item"]);
							SetItemButtonCount(objButton.fraItemButton, objItem["Charges"]);
							WatchFrameItem_UpdateCooldown(objButton.fraItemButton);
							objButton.fraItemButton.rangeTimer = -1;
							
							-- If a button is heigher then its quest then expand the quest frame to stop overlapping buttons
							local intButtonHeight = objButton.fraItemButton:GetHeight() * objButton.fraItemButton:GetScale()
							if (intButtonHeight > intSpacingIncrease) then
								intSpacingIncrease = intButtonHeight
								objButton.objFontString:SetHeight(intButtonHeight)
							end
						end
					end
					objButton:SetHeight(intSpacingIncrease)
					
					intYPosition = intYPosition - intSpacingIncrease
				end
			end
		end
	end

	if (db.Minion.CollapseToLeft == true) then
		local intNonQuestSize = intLargestWidth
		local intQuestSize = intLargestWidth - (intQuestOffset * 2)
	
		if (intNonQuestSize < db.Minion.MinionWidth) then
			fraSQLMainAnchor:SetWidth(intNonQuestSize)
			for i = 1, intCurrentButton, 1 do
				if (dicButtons[strButtonPrefix .. i].isUsed == true) then
					if (dicButtons[strButtonPrefix .. i].isHeader == 1) then
						dicButtons[strButtonPrefix .. i].objFontString:SetWidth(intNonQuestSize)
						dicButtons[strButtonPrefix .. i]:SetWidth(intNonQuestSize)
					else
						dicButtons[strButtonPrefix .. i].objFontString:SetWidth(intQuestSize)
						dicButtons[strButtonPrefix .. i]:SetWidth(intQuestSize)
					end
				end
			end
		end
	end
	blnIsUpdating = false
end

function SorhaQuestLog:GetQuestText(QuestInstance)
	local strLevelColor = ""
	local strTitleColor = ""
	
	-- Get quest level colour
	if (db.ZonesAndQuests.QuestLevelColouringSetting == "Level") then
		local objColour = GetQuestDifficultyColor(QuestInstance["Level"]);	
		strLevelColor = format("|c%02X%02X%02X%02X", 255, objColour.r * 255, objColour.g * 255, objColour.b * 255);
	
	elseif (db.ZonesAndQuests.QuestLevelColouringSetting == "Completion") then
		strLevelColor = self:GetQuestCompletionColour(QuestInstance["CompletionLevel"])
	
	elseif (db.ZonesAndQuests.QuestLevelColouringSetting == "Done/Undone") then
		if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
			strLevelColor = strObjective100Colour
		else
			strLevelColor = strObjective00to24Colour
		end
	
	else
		strLevelColor = strQuestLevelColour
	end
	
	-- Get quest title colour
	if (db.ZonesAndQuests.QuestTitleColouringSetting == "Level") then
		local objColour = GetQuestDifficultyColor(QuestInstance["Level"]);	
		strTitleColor = format("|c%02X%02X%02X%02X", 255, objColour.r * 255, objColour.g * 255, objColour.b * 255);
	
	elseif (db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") then
		strTitleColor = self:GetQuestCompletionColour(QuestInstance["CompletionLevel"])
	
	elseif (db.ZonesAndQuests.QuestTitleColouringSetting == "Done/Undone") then
		if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
			strTitleColor = strObjective100Colour
		else
			strTitleColor = strObjective00to24Colour
		end
	
	else
		strTitleColor = strQuestTitleColour
	end
	
	
	-- Quest level
	local strReturnText = strLevelColor .. "[" .. QuestInstance["Level"]
	
	-- PVP, RAID, Group, etc tags
	if (QuestInstance["Tag"] ~= nil) then
		if (db.ZonesAndQuests.FullLengthQuestTags == true and dicLongQuestTags[QuestInstance["Tag"]] ~= nil) then
			strReturnText = strReturnText .. " " .. dicLongQuestTags[QuestInstance["Tag"]]
		elseif (db.ZonesAndQuests.FullLengthQuestTags == false and dicQuestTags[QuestInstance["Tag"]] ~= nil) then
			strReturnText = strReturnText .. dicQuestTags[QuestInstance["Tag"]]
		end
	end
	
	if (QuestInstance["SuggestedGroup"] > 0) then
		strReturnText = strReturnText .. QuestInstance["SuggestedGroup"]
	end

	if (QuestInstance["IsDaily"]) then
		if (db.ZonesAndQuests.FullLengthQuestTags == true) then
			strReturnText = strReturnText .. " " .. dicLongQuestTags["Daily"]
		else
			strReturnText = strReturnText .. dicQuestTags["Daily"]
		end
	end

	strReturnText = strReturnText .. "]|r " .. strTitleColor .. QuestInstance["Title"] .. "|r"
	
	-- Completion/failed etc tag
	if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] < 0) then 
		strReturnText = strReturnText .. L[" (Failed)"]
	elseif (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
		strReturnText = strReturnText .. L[" (Done)"]
	elseif (numQuestLogLeaderBoards == 0) then
		strReturnText = strReturnText .. L[" (goto)"]
	end
	
	-- Hidden tag
	if ((tContains(dbChar.Quests.HiddenQuests, QuestInstance["ID"]) == 1) and db.ZonesAndQuests.AllowHiddenQuests == true) then
		strReturnText = strReturnText .. strHeaderColour .. L[" (Hidden)"] .. "|r"
	end 
	
	strReturnText = strReturnText .. "\n"
	
	-- Objectives
	if (#QuestInstance["ObjectiveList"] == 0) then
		-- If no objective and show descriptions on display quest description
		if (db.ZonesAndQuests.ShowDescWhenNoObjectives == true) then
			if (not(QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0)) then
				strReturnText = strReturnText .. strObjectiveDescriptionColour .. " - " .. QuestInstance["ObjectiveDescription"] .. "|r\n"
			else
				if (db.ZonesAndQuests.HideCompletedObjectives == false) then
					strReturnText = strReturnText .. strObjectiveDescriptionColour .. " - " .. QuestInstance["ObjectiveDescription"] .. "|r\n"
				end
			end	
		end
	else
		-- For each objective in quest
		for k, ObjectiveInstance in pairs(QuestInstance["ObjectiveList"]) do
			if not(db.ZonesAndQuests.HideCompletedObjectives == true and ObjectiveInstance["Status"] == 1) then
				local strObjectiveGradualColour = "|cffffffff"
				local strObjectiveTitleColourOutput = strObjectiveTitleColour
				local strObjectiveStatusColourOutput = strObjectiveStatusColour
				
				-- If somethings uses gradual colours get colour
				if (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion") then
					strObjectiveGradualColour = self:GetQuestCompletionColour(ObjectiveInstance["CompletionLevel"])
				end
				
				-- Decide on quest title colour
				if (db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion") then
					strObjectiveTitleColourOutput = strObjectiveGradualColour
				elseif (db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Done/Undone") then
					if (ObjectiveInstance["Status"] == nil) then
						strObjectiveTitleColourOutput = strObjective00to24Colour
					else
						strObjectiveTitleColourOutput = strObjective100Colour
					end
				end	

				-- Decide on quest status (0/1 etc) colour
				if (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion") then
					strObjectiveStatusColourOutput = strObjectiveGradualColour
				elseif (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Done/Undone") then
					if (ObjectiveInstance["Status"] == nil) then
						strObjectiveStatusColourOutput = strObjective00to24Colour
					else
						strObjectiveStatusColourOutput = strObjective100Colour
					end
				end				
				
				-- Depending on quest type display it in a certain way
				if (ObjectiveInstance["Type"] == nil) then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"		
					
				elseif (ObjectiveInstance["Type"] == "event") then
					if (ObjectiveInstance["Status"] == nil) then
						strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"
					else
						strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r" .. strObjective100Colour .. " (Done)|r"
					end
					
				elseif (ObjectiveInstance["Type"] == "log") then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"
					
				elseif (ObjectiveInstance["Type"] == "reputation") then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. ": |r" .. strObjectiveStatusColourOutput .. ObjectiveInstance["Got"] .. " / " .. ObjectiveInstance["Need"] .. "|r"	
					
				else
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. ": |r" .. strObjectiveStatusColourOutput .. ObjectiveInstance["Got"] .. "/" .. ObjectiveInstance["Need"] .. "|r"
				end
				
				strReturnText = strReturnText .. strOutput .. "\n"
			end
		end
	end
	strReturnText = strtrim(strReturnText)
	return strReturnText
end

function SorhaQuestLog:GetQuestCompletionColour(dblPercent)
	if (dblPercent < 0.25) then
		return strObjective00to24Colour
	elseif (dblPercent >= 0.25 and dblPercent < 0.50) then
		return strObjective25to49Colour
	elseif (dblPercent >= 0.50 and dblPercent < 0.75) then
		return strObjective50to74Colour
	elseif (dblPercent >= 0.75 and dblPercent < 1) then
		return strObjective75to99Colour
	else
		return strObjective100Colour
	end
end

function SorhaQuestLog:CheckQuestStateChange()
	local blnQuestNotFound = false
	local blnObjectiveCompleted = false
	local blnQuestCompleted = false
	local dicOldQuests = {}
	local dicCurQuests = {}	
	
	-- Compile list of old quests
	for k, ZoneInstance in pairs(oldQuestInfo["Zones"]) do -- Get list of old quests
		for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
			tinsert(dicOldQuests, QuestInstance)
		end
	end	
	
	-- Compile list of new quests
	for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do -- Get list of current quests
		for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
			tinsert(dicCurQuests, QuestInstance)
		end
	end
	
	-- For each quest in the old compare it to each quest in the new.
		-- When they match:
			-- Check each old objective against each new objective to check for status changes
			-- Check old quests status against new quest status
	for oldQuestKey, oldQuestInstance in pairs(dicOldQuests) do
		for curQuestKey, curQuestInstance in pairs(dicCurQuests) do
		
			if (oldQuestInstance["ID"] == curQuestInstance["ID"]) then -- If is same quest
				if (blnObjectiveCompleted == false) then -- If havn't found changed objective look for one
				
					for oldObjectiveKey, oldObjectiveInstance in pairs(oldQuestInstance["ObjectiveList"]) do
						for curObjectiveKey, curObjectiveInstance in pairs(curQuestInstance["ObjectiveList"]) do
						
							if (oldObjectiveInstance["ID"] == curObjectiveInstance["ID"]) then
								if (oldObjectiveInstance["Status"] == nil and curObjectiveInstance["Status"] == 1) then
									blnObjectiveCompleted = true
									break
								end
							end
						end
						if (blnObjectiveCompleted == true) then
							break
						end
					end
				end
				if (oldQuestInstance["IsComplete"] == nil and curQuestInstance["IsComplete"] == 1) then -- Has quest been completed since last
					blnQuestCompleted = true
					break
				end
			end
		end	
		if (blnQuestCompleted == true) then
			break
		end
	end
	
	-- Quest has been completed
	if (blnQuestCompleted == true) then
		return "QuestDone"
	end
	
	-- Objective has been completed
	if (blnObjectiveCompleted == true) then
		return "ObjectiveDone"
	end
	
	return ""
end

function SorhaQuestLog:DisplayRightClickMenu(objButton)
	local objMenu = CreateFrame("Frame", "SorhaQuestLogMenuThing")
	local intLevel = 1
	local info = {}
	
	objMenu.displayMode = "MENU"
	objMenu.initialize = function(self, intLevel)
		if not intLevel then return end
		wipe(info)
		if intLevel == 1 then
			-- Create the title of the menu
			info.isTitle = 1
			info.text = L["Show/Hide Quests"]
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, intLevel)
			
			local intCurrentButton = 0
			local curZone = nil
			
			-- Get zone button belongs to
			for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
				if (ZoneInstance["LogPosition"] == objButton.LogPosition) then
					curZone = ZoneInstance
				end
			end

			-- Show/Hide buttons for each quest
			for k2, QuestInstance in pairs(curZone["Quests"]) do
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = nil
				info.text = QuestInstance["Title"]
				info.func = function()
					if not(tContains(dbChar.Quests.HiddenQuests, QuestInstance["ID"])) then
						tinsert(dbChar.Quests.HiddenQuests, QuestInstance["ID"])	
					else
						SorhaQuestLog:RemoveObject(dbChar.Quests.HiddenQuests, QuestInstance["ID"])
					end
					SorhaQuestLog:UpdateMinion()
				end
				info.checked = not (tContains(dbChar.Quests.HiddenQuests, QuestInstance["ID"]))
				UIDropDownMenu_AddButton(info, intLevel)
			end
	
			-- Hide all button if not all hidden
			if not(curZone["NumHidden"] == curZone["NumQuests"])then
				info.text = L["Hide All"]
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = 1
				info.func = function()
					for k2, QuestInstance in pairs(curZone["Quests"]) do
						if not(tContains(dbChar.Quests.HiddenQuests, QuestInstance["ID"])) then
							tinsert(dbChar.Quests.HiddenQuests, QuestInstance["ID"])
						end
					end
					SorhaQuestLog:UpdateMinion()
				end
				UIDropDownMenu_AddButton(info, intLevel)
			end
			
			-- Show all button if not all hidden
			if (curZone["NumHidden"] > 0)then
				info.text = L["Show All"]
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = 1
				info.func = function()
					for k2, QuestInstance in pairs(curZone["Quests"]) do
						if (tContains(dbChar.Quests.HiddenQuests, QuestInstance["ID"])) then
							SorhaQuestLog:RemoveObject(dbChar.Quests.HiddenQuests, QuestInstance["ID"])
						end
					end
					SorhaQuestLog:UpdateMinion()
				end
				UIDropDownMenu_AddButton(info, intLevel)
			end

			-- Close menu item
			info.text = CLOSE
			info.func = function() CloseDropDownMenus() end
			info.checked = nil
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, intLevel)
		end
	end

	ToggleDropDownMenu(1, nil, objMenu, objButton, 0, 0)
end

function SorhaQuestLog:doHandleZoneChange()
	local blnNewZone = not(strZone == GetRealZoneText())
	strZone = GetRealZoneText()
	strSubZone = GetSubZoneText()
	blnIgnoreUpdateEvents = true
	local blnChanged = false
	
	if (db.ZonesAndQuests.CollapseOnLeave == true or db.ZonesAndQuests.ExpandOnEnter == true) then
		local numEntries, numQuests = GetNumQuestLogEntries();	
		for i = numEntries, 1, -1 do
			local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
			if (isHeader) then
				if (questTitle == strZone or questTitle == strSubZone) then
					if (db.ZonesAndQuests.ExpandOnEnter == true and isCollapsed) then
						ExpandQuestHeader(i)
						blnChanged = true
					end
				else		
					if (db.ZonesAndQuests.CollapseOnLeave == true and not(isCollapsed) and blnNewZone) then
						CollapseQuestHeader(i)
						blnChanged = true
					end				
				end
			end
		end
	end

	blnIgnoreUpdateEvents = false
	if (blnInitialized == true and blnChanged == true) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:SetupOptions() --Creates options. Registers chat commands
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SorhaQuestLog", getOptions())
	
	AceConfigDialog:SetDefaultSize("SorhaQuestLog", 680,525)
	self:RegisterChatCommand("sql", "ChatCommand")
	self:RegisterChatCommand("SorhaQuestLog", "ChatCommand")
end

function SorhaQuestLog:ChatCommand(input) --Handles chat commands
	if not input or input:trim() == "" then
		LibStub("AceConfigDialog-3.0"):Open("SorhaQuestLog")
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SorhaQuestLog, "SorhaQuestLog", "SorhaQuestLog", input)
	end
end

function SorhaQuestLog:doItemButtonSizeChange() --Changes the addons colours on the fly as changed
	for i=1,intNumberOfFrames,1 do
		dicButtons[strButtonPrefix .. i].fraItemButton:SetScale(db.Minion.ItemButtonScale)
	end
	self:UpdateMinion()
end

function SorhaQuestLog:doColourChange() --Changes the addons colours on the fly as changed
	self:SetupColours()
	if (blnIsUpdating == false) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:SetupColours() -- Sets the colour strings from profile settings
	strInfoColour = format("|c%02X%02X%02X%02X", 255, db.Colours.InfoColour.r * 255, db.Colours.InfoColour.g * 255, db.Colours.InfoColour.b * 255);
	strHeaderColour = format("|c%02X%02X%02X%02X", 255, db.Colours.HeaderColour.r * 255, db.Colours.HeaderColour.g * 255, db.Colours.HeaderColour.b * 255);
	strQuestLevelColour = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestLevelColour.r * 255, db.Colours.QuestLevelColour.g * 255, db.Colours.QuestLevelColour.b * 255);
	strQuestTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestTitleColour.r * 255, db.Colours.QuestTitleColour.g * 255, db.Colours.QuestTitleColour.b * 255);
	strObjectiveTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveTitleColour.r * 255, db.Colours.ObjectiveTitleColour.g * 255, db.Colours.ObjectiveTitleColour.b * 255);
	strObjectiveStatusColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveStatusColour.r * 255, db.Colours.ObjectiveStatusColour.g * 255, db.Colours.ObjectiveStatusColour.b * 255);
	strObjective00to24Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective00PlusColour.r * 255, db.Colours.Objective00PlusColour.g * 255, db.Colours.Objective00PlusColour.b * 255);
	strObjective25to49Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective25PlusColour.r * 255, db.Colours.Objective25PlusColour.g * 255, db.Colours.Objective25PlusColour.b * 255);
	strObjective50to74Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective50PlusColour.r * 255, db.Colours.Objective50PlusColour.g * 255, db.Colours.Objective50PlusColour.b * 255);
	strObjective75to99Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective75PlusColour.r * 255, db.Colours.Objective75PlusColour.g * 255, db.Colours.Objective75PlusColour.b * 255);
	strObjective100Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveDoneColour.r * 255, db.Colours.ObjectiveDoneColour.g * 255, db.Colours.ObjectiveDoneColour.b * 255);
	strObjectiveDescriptionColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveDescColour.r * 255, db.Colours.ObjectiveDescColour.g * 255, db.Colours.ObjectiveDescColour.b * 255);
end

function SorhaQuestLog:HandleBlizzardTracker()
	if (db.Main.HideBlizzardTracker == true) then
		WatchFrame:Hide()
    else
		WatchFrame:Show()
    end
end

function SorhaQuestLog:CleanupHiddenQuestList() --Clean out completed/abandoned quests from hidden quests table
	blnIgnoreUpdateEvents = true
	
	local haveQuests = {}
	local dicHeaders = {}
	local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID
	local numEntries, numQuests = GetNumQuestLogEntries();	
	
	-- For each header store its current state, and if collapsed expand it.
	for i = numEntries, 1, -1 do
		questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if (isHeader) then
			if not(questTitle == nil) then
				if (isCollapsed) then
					dicHeaders[questTitle] = true
					ExpandQuestHeader(i)
				else
					dicHeaders[questTitle] = false
				end
			end
		end
	end
	
	-- For each quest we have insert it into a table of quests we have
	numEntries, numQuests = GetNumQuestLogEntries();	
	for i = 1, numEntries, 1 do
		questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if (not isHeader) then
			tinsert(haveQuests, questID)
		end
	end
	
	-- For each of the quests we have check if it is in the hidden table, if so insert it to a temp table which will become the new HiddenQuests table
	local tempTable = {}
	for key, qID in pairs(haveQuests) do
		if (tContains(dbChar.Quests.HiddenQuests, qID) == 1) then
			tinsert(tempTable, qID)
		end
	end
	dbChar.Quests.HiddenQuests = tempTable 
	
	-- For each header if it was collapsed originally recollapse it now
	for i = numEntries, 1, -1 do
		questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if (isHeader) then
			if not(questTitle == nil) then
				if (dicHeaders[questTitle] == true) then
					CollapseQuestHeader(i)
				end
			end
		end
	end
	blnIgnoreUpdateEvents = false
end

function SorhaQuestLog:RemoveObject(objTable, objFind)
	for k, v in pairs(objTable) do
		if (v == objFind) then
			tremove(objTable,k)
			break
		end
	end
end

function SorhaQuestLog:doAnchorUpdates(blnMoveAnchors) -- Updates backgrounds and visibility of anchor frames
	if (db.Main.HideSorhaQuestLog == true) then
		fraSQLMainAnchor:Hide()
	else
		fraSQLMainAnchor:Show()
	end
	
	if (blnMoveAnchors == true) then
		fraSQLMainAnchor:ClearAllPoints()
		fraSQLMainAnchor:SetPoint(db.Minion.MinionWindowLocation.Point, UIParent, db.Minion.MinionWindowLocation.RelativePoint, db.Minion.MinionWindowLocation.X, db.Minion.MinionWindowLocation.Y);
	end
	
	if (db.Minion.MinionWindowLocked == false) then
		fraSQLMainAnchor:SetBackdropColor(0, 1, 0, 0.5)
		fraSQLMainAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
		fraSQLMainAnchor:EnableMouse(true)
	else
		fraSQLMainAnchor:SetBackdropColor(0, 1, 0, 0)
		fraSQLMainAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
		fraSQLMainAnchor:EnableMouse(false)
	end
	
	
	if (blnTimerInitialized == true) then
		if (db.Main.ShowQuestTimerFrame == true) then
			fraQuestTimerAnchor:Show()
		else
			fraQuestTimerAnchor:Hide()
		end
		
		if (blnMoveAnchors == true) then
			fraQuestTimerAnchor:ClearAllPoints()
			fraQuestTimerAnchor:SetPoint(db.Timer.TimerWindowLocation.Point, UIParent, db.Timer.TimerWindowLocation.RelativePoint, db.Timer.TimerWindowLocation.X, db.Timer.TimerWindowLocation.Y);
		end
			
		if (db.Timer.TimerWindowLocked == false) then
			fraQuestTimerAnchor:SetBackdropColor(0, 1, 0, 0.5)
			fraQuestTimerAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
			fraQuestTimerAnchor:EnableMouse(true)
		else
			fraQuestTimerAnchor:SetBackdropColor(0, 1, 0, 0)
			fraQuestTimerAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
			fraQuestTimerAnchor:EnableMouse(false)
		end
	end
end

function SorhaQuestLog:doHiddenQuestsUpdate()
	-- Show/Hide hidden quests button and move quest count text accordingly
	if (db.ZonesAndQuests.AllowHiddenQuests == true) then
		fraSQLMainAnchor.buttonShowHidden:Show()	
		fraSQLMainAnchor.buttonShowHiddenHover:Show()
		fraSQLMainAnchor.objFontString:ClearAllPoints()
		fraSQLMainAnchor.objFontString:SetPoint("TOPLEFT", fraSQLMainAnchor, "TOPLEFT", 16, -5);
		if (db.ZonesAndQuests.ShowAllQuests == true) then
			fraSQLMainAnchor.buttonShowHidden:SetBackdropColor(0, 0.3, 1, 1)
		else
			fraSQLMainAnchor.buttonShowHidden:SetBackdropColor(0.5, 0.5, 0.5, 1)
		end
	else
		fraSQLMainAnchor.buttonShowHidden:Hide()	
		fraSQLMainAnchor.buttonShowHiddenHover:Hide()
		fraSQLMainAnchor.objFontString:ClearAllPoints()
		fraSQLMainAnchor.objFontString:SetPoint("TOPLEFT", fraSQLMainAnchor, "TOPLEFT", 0, -5);
	end
	if (blnInitialized == true) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:doCreateLooseFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,intAlpha,finherit) -- Returns a frame thats not anchored
	local f = CreateFrame(strType,strName,fraParent,finherit)
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)	
	f:SetAlpha(intAlpha)
	return f  
end 

function SorhaQuestLog:doCreateFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY,intAlpha,finherit) -- Returns an anchored frame
	local f = CreateFrame(strType,strName,fraParent,finherit)
	
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)
	f:SetPoint(strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY)
	f:SetAlpha(intAlpha)
	return f  
end 

function SorhaQuestLog:ShowSorhaQuestLog()
	db.Main.HideSorhaQuestLog = false
	fraSQLMainAnchor:Show()
	if (blnInitialized == true and blnIsUpdating == false) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:HideSorhaQuestLog()
	db.Main.HideSorhaQuestLog = true
	fraSQLMainAnchor:Hide()
end

function SorhaQuestLog:ToggleSorhaQuestLog()
	if (db.Main.HideSorhaQuestLog == true) then
		self:ShowSorhaQuestLog()
	else
		self:HideSorhaQuestLog()
	end
end

function SorhaQuestLog:GetQuestLogInformation()
	local intNoEntries, intNoQuests = GetNumQuestLogEntries();
	local i = 1
	local intEntrys = 0
	local QuestLogInfo = {["NoEntrys"] = 0, ["NoQuests"] = intNoQuests, ["Zones"] = {}}
	local ZoneEntry = nil
	local QuestEntry = {}
	local ObjectiveEntry = {}
	local ItemEntry = {}
	
	local intNumHidden = 0
	local intNumQuestsInZone = 0
	
	-- For each entry in the quest log
	for i = 1, intNoEntries, 1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if (questTitle == nil) then
			questTitle = L["Unknown"]
		end
		
		if (isHeader) then
			-- Create new zone entry
			intEntrys = intEntrys + 1
			if not(ZoneEntry == nil) then
				ZoneEntry["NumHidden"] = intNumHidden
				ZoneEntry["NumQuests"] = intNumQuestsInZone
				tinsert(QuestLogInfo["Zones"], ZoneEntry)
				intNumHidden = 0
				intNumQuestsInZone = 0
			end
			ZoneEntry = {["Title"] = questTitle, ["Status"] = isCollapsed, ["LogPosition"] = i, ["ID"] = questID, ["HasHidden"] = false, ["NumHidden"] = intNumHidden, ["NumQuests"] = intNumQuestsInZone, ["Quests"] = {}}
		else
			-- Create new zone
			intNumQuestsInZone = intNumQuestsInZone + 1
			intEntrys = intEntrys + 1
			
			QuestEntry = {["Title"] = questTitle, ["Level"] = level, ["Tag"] = questTag, ["IsHidden"] = false, ["SuggestedGroup"] = suggestedGroup, ["IsComplete"] = isComplete, ["IsDaily"] = isDaily, ["LogPosition"] = i, ["ID"] = questID, ["QuestItem"] = nil, ["CompletionLevel"] = 0, ["ObjectiveList"] = {}, ["ObjectiveDescription"] = ""}
			
			-- Hidden quest settings
			if (tContains(dbChar.Quests.HiddenQuests, questID)) then
				intNumHidden = intNumHidden + 1
				QuestEntry["IsHidden"] = true
				ZoneEntry["HasHidden"] = true
			end
			
			-- Quest item
			local strLink, strItem, intCharges = GetQuestLogSpecialItemInfo(i);
			if (strItem) then
				ItemEntry = {["Link"] = strLink, ["Item"] = strItem, ["Charges"] = intCharges}
				QuestEntry["QuestItem"] = ItemEntry
			end
			
			SelectQuestLogEntry(i);
			local strQuestDescription, strQuestObjectiveDescription = GetQuestLogQuestText();
			local intNoOfObjectives = GetNumQuestLeaderBoards(i)
			local dblQuestCompletionPercent = 0
			local intNumUsedObjectives = 0
			
			QuestEntry["ObjectiveDescription"] = strQuestObjectiveDescription
			
			-- Objectives
			for k = 1, intNoOfObjectives, 1 do
				local leaderboardTxt, itemType, isDone = GetQuestLogLeaderBoard(k,i);
				if not(leaderboardTxt == nil) then
					intNumUsedObjectives = intNumUsedObjectives + 1
					local strObjText = ""
					local strGot = ""
					local strGetTo = ""
					local intCompletion = 0
					local intGot = 0
					local intNeeded = 1
				
					-- Change things based on objective type
					if (itemType == nil) then
						strObjText = leaderboardTxt
						
					elseif (itemType == "event") then
						strObjText = leaderboardTxt
						
					elseif (itemType == "log") then
						strObjText = leaderboardTxt		
						
					elseif (itemType == "reputation") then
						local y, z, strObjDesc, strRepHave, strRepNeeded = string.find(leaderboardTxt, "(.*):%s*(.*)%s*/%s*(.*)");
						if (strObjDesc == nil) then strObjText = "" else strObjText = strObjDesc end
						if (strRepHave == nil) then strGot = "" else strGot = strtrim(strRepHave) end
						if (strRepNeeded == nil) then strGetTo = "" else strGetTo = strtrim(strRepNeeded) end
						
						for k, RepInstance in pairs(dicRepLevels) do
							if (strGot == RepInstance["MTitle"] or strGot == RepInstance["FTitle"]) then
								intGot = RepInstance["Value"]
							end
							if (strGetTo == RepInstance["MTitle"] or strGetTo == RepInstance["FTitle"]) then
								intNeeded = RepInstance["Value"]
							end
						end
						intCompletion = intGot / intNeeded
					
					else
						local y, z, strObjDesc, intNumGot, intNumNeeded = string.find(leaderboardTxt, "(.*):%s*([%d]+)%s*/%s*([%d]+)");
						if (strObjDesc == nil) then strObjText = "" else strObjText = strObjDesc end
						if (intNumGot == nil) then strGot = 2 else strGot = intNumGot end
						if (intNumNeeded == nil) then strGetTo = 1 else strGetTo = intNumNeeded end
						intCompletion = strGot / strGetTo
					end
		
					if (isDone) then
						intCompletion = 1
					end
					dblQuestCompletionPercent = dblQuestCompletionPercent + intCompletion
					ObjectiveEntry = {["Text"] = strObjText, ["Got"] = strGot, ["Need"] = strGetTo, ["CompletionLevel"] = intCompletion, ["Type"] = itemType, ["Status"] = isDone, ["ID"] = k}
					tinsert(QuestEntry["ObjectiveList"], ObjectiveEntry)
				end
			end
			if (intNoOfObjectives < 1) then
				dblQuestCompletionPercent = 1
			else
				dblQuestCompletionPercent = dblQuestCompletionPercent / intNumUsedObjectives
			end
			QuestEntry["CompletionLevel"] = dblQuestCompletionPercent
			tinsert(ZoneEntry["Quests"], QuestEntry)
		end
	end
	if (ZoneEntry ~= nil) then
		ZoneEntry["NumHidden"] = intNumHidden
		ZoneEntry["NumQuests"] = intNumQuestsInZone
		tinsert(QuestLogInfo["Zones"], ZoneEntry)
	end
	QuestLogInfo["NoEntrys"] = intEntrys
	
	return QuestLogInfo
end